/*
 * $Id: common.c,v 1.3 2002/01/08 01:49:40 tzafrir Exp $
 */
#include <X11/Xlib.h>

#define FRIBIDI_0_9_INTERFACE

#include <fribidi/fribidi.h>
#include <r2llib.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
 
#include "parse_options.h"
#include "hook_into_x.h"


/* the options we're running with */
static biditext_options_t options;

/* the pointers into X (defined in biditext.c)*/
extern x_funcs_t x_funcs;

/* the r2llib opaque token we're working with */
static r2llib_t rtl;



/* the maximum buffer size */
#define BIDITEXT_STR_BUFF_SIZE 4096

/* profiling related stuff */
#ifndef BIDITEXT_NO_PROFILE
static struct timeval tv1, tv2;
static char* msg;
#define START_PROFILING(x) \
     if (options.EnableProfiling) { \
         msg = (x); \
         gettimeofday(&tv1, NULL);\
     }
#define REPORT_PROFILING \
     if (options.EnableProfiling) {\
         gettimeofday(&tv2, NULL); \
         fprintf(stderr, "%s: %ld.%6ld\n", msg, \
                 tv2.tv_sec - tv1.tv_sec - (tv2.tv_usec - tv1.tv_usec < 0),  \
	         (tv2.tv_usec - tv1.tv_usec) + (1000000 * (tv2.tv_usec - tv1.tv_usec < 0)));\
     }
#else
#define START_PROFILING(x)
#define REPORT_PROFILING
#endif /* BIDITEXT_NO_PROFILE */ 


/* functions */

static inline int 
is_bidi_charset(const char* registry, const char* encoding)
{
     return (registry && encoding &&
	     (options.LangHebrew &&
	        ((strncasecmp(registry,"iso8859",7)==0 && 
	          strncasecmp(encoding,"8",1)==0) ||
	         (strncasecmp(registry,"microsoft",9)==0 && 
	          strncasecmp(encoding,"cp1255",6)==0) ||
	         (strncasecmp(registry,"iso10646",8)==0 && 
	          strncasecmp(encoding,"1",1)==0) ))
               )/* || (LangArabic && (...))*/;
}

static inline FriBidiCharType
get_base_bidi_dir(BIDITEXT_BASE_STATE bt)
{
     switch(bt){
     case (BT_BASE_LTR):
	  return FRIBIDI_TYPE_LTR;
	  break; /* not reached */
     case (BT_BASE_RTL):
	  return FRIBIDI_TYPE_RTL;
	  break; /* not reached */
     case (BT_BASE_NEUTRAL): /* fall through */
     case (BT_BASE_ERROR):   /* fall through */
     default: 
	  return FRIBIDI_TYPE_ON;
	  break; /* not reached */
     }
}

static inline int
should_do_biditext(Display* dpy, GC gc, r2llib_t* prtl)
{
     /* used for retrieving font properties */
     char *font_charset_encoding = 0;
     char *font_charset_registry = 0;
     unsigned long prop_value;
     static Atom atom_charset_registry = 0;
     static Atom atom_charset_encoding = 0;
     int perform_bidi = 0;
     static Font last_font_non_bidi = 0; /* prevent possible race condition -- assuming font ids don't repeat */
     static Font last_font_bidi = 0;
     XGCValues gc_values;
     
     XFontStruct *xfont;

     if (!prtl)
	  return 0;

     /* if r2llib is not initialized yet, initialize it now */
     if (!rtl){
	  rtl = r2l_init(0, NULL, NULL);

	  /* if we failed to init r2llib, bail out */
	  if (!rtl) {
	       return 0;
          }
          /* Other one-time initilizations */
	  biditext_parse_options(&options);
	  biditext_hook_into_x(&x_funcs);
     }

     /* make prtl point to our r2l handle */
     *prtl = rtl;
     
     if (! options.EnableCharsetCheck) { /* Skip the charsets check altogether */
          return 1;
     }
     
     /* retrieve X atoms, if needed */
     if (!atom_charset_registry) { 
          atom_charset_registry = XInternAtom(dpy, "CHARSET_REGISTRY", 1); 
     }
     if (!atom_charset_encoding) { 
          atom_charset_encoding = XInternAtom(dpy, "CHARSET_ENCODING", 1); 
     }

     /* retrieve the font properties (charset, encoding) */
     if (XGetGCValues(dpy, gc, GCFont, &gc_values)) {
	  if (last_font_bidi == gc_values.font) {
	       perform_bidi = 1;
	  } else if (last_font_non_bidi == gc_values.font) {
	       perform_bidi = 0;
	  } else {
	       if ((xfont = XQueryFont(dpy, XGContextFromGC(gc)))) {
		    if (XGetFontProperty(xfont, atom_charset_encoding, 
                                            &prop_value)) {
			 font_charset_encoding = 
                                 XGetAtomName(dpy, (Atom) prop_value);
		    }
		    if (XGetFontProperty(xfont, atom_charset_registry, 
                                            &prop_value)) {
			 font_charset_registry = 
                                 XGetAtomName(dpy, (Atom) prop_value);
		    }
		    /* decide whether to perform BiDi */
                    /* for the given character set    */
                    perform_bidi = is_bidi_charset(font_charset_registry, 
						   font_charset_encoding);
		    
		    free(font_charset_registry);
		    free(font_charset_encoding);
		    XFreeFontInfo(0, xfont, 1);
	     
		    if (perform_bidi)
			 last_font_bidi = gc_values.font;
		    else
			 last_font_non_bidi = gc_values.font;
	       }
	  }
     }
     
     return perform_bidi;
}

XChar2b*
do_biditext16(Display* dpy, GC gc, XChar2b* string, int length)
{
     static XChar2b string_final[BIDITEXT_STR_BUFF_SIZE];
     r2llib_t rtl;

     START_PROFILING("do_biditext16: ");
     if ((length < BIDITEXT_STR_BUFF_SIZE) && should_do_biditext(dpy, gc, &rtl)){
	  R2L_STATE st = R2L_ERROR;
	  BIDITEXT_BASE_STATE bt = BT_BASE_ERROR;

	  /* get the current r2l state and bt state */
	  r2l_query_state(rtl, &st, &bt);

	  if (R2L_ENABLED == st){
	       //guint16 positionLtoV[BIDITEXT_STR_BUFF_SIZE], positionVtoL[BIDITEXT_STR_BUFF_SIZE];
	       FriBidiChar string_visual[BIDITEXT_STR_BUFF_SIZE], string_logical[BIDITEXT_STR_BUFF_SIZE];
	       FriBidiCharType base;
	       //char el[BIDITEXT_STR_BUFF_SIZE];
	       int i;

	       /* XDrawString16 expects UCS-2 Big Endian -- FriBidi expects UCS-2 Little Endian */
	       for(i=0; i<length; i++) {
		    string_logical[i] = (string[i].byte1 << 8) + string[i].byte2;
	       }
	       
	       /* Create a bidi string */
	       base = get_base_bidi_dir(bt);

	       /*base = FRIBIDI_TYPE_L;*/
	       fribidi_log2vis(string_logical, length, &base,
			       /* output */
			       string_visual,
			       NULL,//positionLtoV,
			       NULL,//positionVtoL,
			       NULL//el
		    );
	       
	       /* Return original endianess and convert the guint32 back to XChar2b */
	       for(i=0; i<length; i++) {
		    string_final[i].byte1 = string_visual[i] >> 8;
		    string_final[i].byte2 = string_visual[i];
	       }

	       REPORT_PROFILING;
	       return string_final;
	  }
     }
     return NULL;
}


char*
do_biditext8(Display* dpy, GC gc, char* string, int length)

{
     static char string_final[BIDITEXT_STR_BUFF_SIZE];
     r2llib_t rtl; 

     START_PROFILING("do_biditext8: ");
     if (options.TraceAnyCall) { /* The first one may be missed, as it is before the */
                         /* call to biditext_init                            */
          fprintf(stderr,"biditext trace: called.          string: \"%s\"\n",string);
     }
     if ((length < BIDITEXT_STR_BUFF_SIZE) && should_do_biditext(dpy, gc, &rtl)){
	  R2L_STATE st = R2L_ERROR;
	  BIDITEXT_BASE_STATE bt = BT_BASE_ERROR;

          if (options.TraceCorrectCharset) { 
               fprintf(stderr,"biditext trace: correct charset. string: \"%s\"\n",string);
          }
	  /* get the current r2l state and bt state */
	  r2l_query_state(rtl, &st, &bt);

	  if (R2L_ENABLED == st){
	       char string_original[BIDITEXT_STR_BUFF_SIZE];//, 
                    //el[BIDITEXT_STR_BUFF_SIZE];
	       FriBidiChar string_visual[BIDITEXT_STR_BUFF_SIZE], 
                           string_logical[BIDITEXT_STR_BUFF_SIZE];
	       //guint16 positionLtoV[BIDITEXT_STR_BUFF_SIZE], 
               //        positionVtoL[BIDITEXT_STR_BUFF_SIZE];
	       FriBidiCharType base = get_base_bidi_dir(bt);

               if (options.TraceLogToVis) { 
                    fprintf(stderr,"biditext trace: converting.      string: \"%s\"\n",string);
               }
	       strncpy(string_original, string, length);
	       string_original[length] = 0;
#ifdef FRIBIDI_INTERFACE_1
	       fribidi_iso8859_8_to_unicode(string_original, string_visual);
#else
	       fribidi_charset_to_unicode(FRIBIDI_CHARSET_ISO8859_8, 
			       string_original, length, string_visual);
#endif /* defined BIDITEXT_INTERFACE_1 */
	       fribidi_log2vis(string_visual, length, &base, 
			       string_logical, 
			       NULL,//positionLtoV, 
			       NULL,//positionVtoL, 
			       NULL);//el);
#ifdef FRIBIDI_INTERFACE_1    
	       fribidi_unicode_to_iso8859_8(string_logical, length, string_final);
#else
	       fribidi_unicode_to_charset(FRIBIDI_CHARSET_ISO8859_8, 
			       string_logical, length, string_final);
#endif /* defined BIDITEXT_INTERFACE_1 */	       
               if (options.TraceLogToVis) { 
                    fprintf(stderr,"biditext trace: after log2vis    string: \"%s\"\n",string_final);
               }
 	       REPORT_PROFILING;
	       return string_final;
	  }
     }
     
     return NULL;
}
