/*
 * $Id: parse_options.c,v 1.1.1.1 2001/09/13 12:42:16 mulix Exp $
 */
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <assert.h>

#include "parse_options.h"

#define BIDITEXT_OPTIONS_ENV_STR "BIDITEXT_OPTIONS"

void
biditext_parse_options(biditext_options_t* opts)
{
     char* options_str;

     assert(opts);
     
     /* parse options string                             */
     /* options_str is a comma-seperated list of options */
     options_str = getenv(BIDITEXT_OPTIONS_ENV_STR);
     if (options_str != NULL && opts) {
          char* curr_ptr = options_str;
          char* next_ptr;
          unsigned int len;
          
	  fprintf(stderr, "options str: '%s'\n", options_str);

	  /* set the defaults: */
	  opts->LangHebrew = 1; 
	  opts->EnableCharsetCheck = 1;

          do {
               next_ptr = strchr(curr_ptr,',');
               if (next_ptr == NULL) { /* no more commas */
                    next_ptr = strchr(curr_ptr,'\0'); /* The end of the string */
               }
               len = next_ptr - curr_ptr -1;

               /* Check the options. The order of the options matters */
	       if (strncmp(curr_ptr,"PROFILE",len) == 0){
                    opts->EnableProfiling = 1;
               } 
               else if (strncmp(curr_ptr,"NO_PROFILE",len) == 0){
                    opts->EnableProfiling = 0;
               } 
               else if (strncmp(curr_ptr,"CHECK_CHARSET",len) == 0) {
                    opts->EnableCharsetCheck = 1;
               }
               else if (strncmp(curr_ptr,"NO_CHECK_CHARSET",len) == 0) {
                    opts->EnableCharsetCheck = 0;
               }
               else if (strncmp(curr_ptr,"TRACE_ANY",len) == 0) {
                    opts->TraceAnyCall = 1;
                    opts->TraceCorrectCharset = 1;
                    opts->TraceLogToVis = 1;
               }
               else if (strncmp(curr_ptr,"TRACE_CHARSET",len) == 0) {
                    opts->TraceAnyCall = 0;
                    opts->TraceCorrectCharset = 1;
                    opts->TraceLogToVis = 1;
               }
               else if (strncmp(curr_ptr,"TRACE_LOG2VIS",len) == 0) {
                    opts->TraceAnyCall = 0;
                    opts->TraceCorrectCharset = 0;
                    opts->TraceLogToVis = 1;
               }
               else if (strncmp(curr_ptr,"NO_TRACE",len) == 0) {
                    opts->TraceAnyCall = 0;
                    opts->TraceCorrectCharset = 0;
                    opts->TraceLogToVis = 0;
               }
               else if (strncmp(curr_ptr,"DUMP",len) == 0) {
                    /* For debugging: print current (not final) options */
                    fprintf(stderr,"biditext: EnableProfiling=%d\n",
                                    opts->EnableProfiling);
                    fprintf(stderr,"biditext: EnableCharsetCheck=%d\n",
                                    opts->EnableCharsetCheck);
                    fprintf(stderr,"biditext: TraceAnyCall=%d\n",opts->TraceAnyCall);
                    fprintf(stderr,"biditext: TraceCorrectCharset=%d\n",
			            opts->TraceCorrectCharset);
                    fprintf(stderr,"biditext: TraceLogToVis=%d\n",
			            opts->TraceLogToVis);
               }
               curr_ptr = next_ptr + 1;
          } while (*next_ptr != '\0');
     }
}
