/* ui_plugin_gnome_applet.c - implementation of a gnome-applet plugin for r2l. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <applet-widget.h>

#include "ui_plugin.h"

/* gnome-related application definitions. */
#define GNOME_APPLET_PACKAGE "gnome_applet_r2l"
#define GNOME_APPLET_VERSION "0.1"

/* plugin's data structure. */
struct ui_plugin_data {
    char* progname;		/* name of the program we're pluged into.  */
    char* title;		/* title of the applet.                    */
    int thin_interface;		/* should we use a thin interface or not?  */
    r2llib_t r2l_ctrl;	 	/* r2l controller.                         */
    R2L_STATE md;	 	/* last known r2l mode.                    */
    BIDITEXT_BASE_STATE bs;	/* last known biditext base mode.          */
    GtkWidget* applet;	 	/* the applet's widget.                    */
    GtkWidget* r2l_md_toggle;   /* button to toggle enable/disable staet.  */
    GtkWidget* r2l_bs_toggle;   /* button to toggle bidi base state.       */
    GdkPixmap* gdk_pixmap_ln;   /* pixmaps used to show r2l's state.       */
    GdkBitmap* gdk_mask_ln;
    GdkPixmap* gdk_pixmap_ll;
    GdkBitmap* gdk_mask_ll;
    GdkPixmap* gdk_pixmap_lr;
    GdkBitmap* gdk_mask_lr;
    GdkPixmap* gdk_pixmap_rn;
    GdkBitmap* gdk_mask_rn;
    GdkPixmap* gdk_pixmap_rl;
    GdkBitmap* gdk_mask_rl;
    GdkPixmap* gdk_pixmap_rr;
    GdkBitmap* gdk_mask_rr;
    GtkPixmap* pixmap_state;
    guint timer;	 	/* timer registered by application.        */
};

/* forward declaration. */
void r2l_md_toggle_cb(GtkToggleButton* r2l_toggle, void* data);
void r2l_bs_toggle_cb(GtkToggleButton* r2l_toggle, void* data);

#include "ln.xpm"
#include "ll.xpm"
#include "lr.xpm"
#include "rn.xpm"
#include "rl.xpm"
#include "rr.xpm"

/* initialize a given client. returns a ui_plugin_t opaque pointer. */
ui_plugin_t
ui_init(int argc,
	char* argv[],
	r2llib_t r2l_ctrl,
	char* title,
	int thin_interface)
{
    ui_plugin_t ui = (ui_plugin_t)malloc(sizeof(struct ui_plugin_data));
    GtkWidget* controls;
    GtkWidget* box;
    GtkWidget* r2l_title;
    GtkWidget* cbox;
    GtkWidget* md_box;
    GtkWidget* bs_box;

    if (!ui)
	goto ui_init_ret;

    ui->r2l_ctrl = r2l_ctrl;
    ui->md = R2L_ERROR;
    ui->bs = BT_BASE_ERROR;
    ui->thin_interface = thin_interface;
    ui->progname = (char*)malloc(strlen(argv[0])+1);
    if (!ui->progname)
	goto err_free_ui_plugin;
    strcpy(ui->progname, argv[0]);
    ui->title = (char*)malloc(strlen(title)+1);
    if (!ui->title)
	goto err_free_progname;
    strcpy(ui->title, title);
    ui->applet = NULL;
    ui->pixmap_state = NULL;
    ui->timer = 0;

    /* initialize a gnome applet. */
    applet_widget_init(GNOME_APPLET_PACKAGE,
		       GNOME_APPLET_VERSION,
		       argc,
		       argv,
                       NULL,
		       0,
		       NULL);
    ui->applet = applet_widget_new(GNOME_APPLET_PACKAGE);
    if (!ui)
	goto err_free_title;

    /* create controls frame */
    /*controls = gtk_frame_new("");*/
    /*gtk_widget_show(controls);*/

    /* create an v. box in the controls frame */
    box = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(box);
    /*gtk_container_add(GTK_CONTAINER(controls), box);*/
    controls = box;

    if (ui->title && ui->title[0]) {
        /* create a title and place in the v. box. */
        r2l_title = gtk_label_new(title);
        gtk_widget_show(r2l_title);
        gtk_container_add(GTK_CONTAINER(box), r2l_title);
    }
    
    /* create an h. box in the controls v. box */
    cbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(cbox);
    gtk_container_add(GTK_CONTAINER(box), cbox);

    /* create the r2l mode controlls in a vbox, placed in the cbox. */
    md_box = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(md_box);
    gtk_container_add(GTK_CONTAINER(cbox), md_box);
    if (ui->thin_interface) {
    	ui->r2l_md_toggle = gtk_button_new_with_label(" ");
    }
    else {
    	ui->r2l_md_toggle = gtk_button_new_with_label("on/off");
    }
    gtk_widget_show(ui->r2l_md_toggle);
    gtk_box_pack_start(GTK_BOX(md_box), ui->r2l_md_toggle, FALSE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(ui->r2l_md_toggle),
		       "clicked",
		       r2l_md_toggle_cb,
		       ui);

    /* create the biditext base mode controlls in a vbox, placed in the cbox. */
    bs_box = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(bs_box);
    gtk_container_add(GTK_CONTAINER(cbox), bs_box);
    if (ui->thin_interface) {
    	ui->r2l_bs_toggle = gtk_button_new_with_label(" ");
    }
    else {
    	ui->r2l_bs_toggle = gtk_button_new_with_label("bidi state");
    }
    gtk_widget_show(ui->r2l_bs_toggle);
    gtk_box_pack_start(GTK_BOX(bs_box), ui->r2l_bs_toggle, FALSE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(ui->r2l_bs_toggle),
		       "clicked",
		       r2l_bs_toggle_cb,
		       ui);

    if (!thin_interface) {
        /* create pixmaps for showing r2l's state. */
        ui->gdk_pixmap_ln = gdk_pixmap_create_from_xpm_d(ui->applet->window,
							 &ui->gdk_mask_ln,
							 NULL,
							 (gchar**)&ln_xpm);
        ui->gdk_pixmap_ll = gdk_pixmap_create_from_xpm_d(ui->applet->window,
							 &ui->gdk_mask_ll,
							 NULL,
							 (gchar**)&ll_xpm);
        ui->gdk_pixmap_lr = gdk_pixmap_create_from_xpm_d(ui->applet->window,
							 &ui->gdk_mask_lr,
							 NULL,
							 (gchar**)&lr_xpm);
        ui->gdk_pixmap_rn = gdk_pixmap_create_from_xpm_d(ui->applet->window,
							 &ui->gdk_mask_rn,
							 NULL,
							 (gchar**)&rn_xpm);
        ui->gdk_pixmap_rl = gdk_pixmap_create_from_xpm_d(ui->applet->window,
							 &ui->gdk_mask_rl,
							 NULL,
							 (gchar**)&rl_xpm);
        ui->gdk_pixmap_rr = gdk_pixmap_create_from_xpm_d(ui->applet->window,
				                         &ui->gdk_mask_rr,
							 NULL,
							 (gchar**)&rr_xpm);
        ui->pixmap_state =
	    (GtkPixmap*)gtk_pixmap_new(ui->gdk_pixmap_ln, ui->gdk_mask_ln);
        gtk_box_pack_start(GTK_BOX(box),
			   GTK_WIDGET(ui->pixmap_state),
			   FALSE,
			   TRUE,
			   0);
        gtk_widget_show(GTK_WIDGET(ui->pixmap_state));
    }


    applet_widget_add(APPLET_WIDGET(ui->applet), controls);
    gtk_widget_show(ui->applet);

    if (!ui->thin_interface) {
        fprintf(stdout, "%s: Gnome-Applet Plugin Initialized\n", ui->progname);
        fflush(stdout);
    }

    goto ui_init_ret;

err_free_title:
    free(ui->title);
    ui->title = NULL;
    
err_free_progname:
    free(ui->progname);
    ui->progname = NULL;
    
err_free_ui_plugin:
    free(ui);
    ui = NULL;
    
ui_init_ret:
    return ui;
}

/* define whether the ui plugin runs its own main loop. */
/* returns '1' for true, '0' for false.                 */
/* note that if it does, it must also support the timer */
/* registration interface.                              */
int
ui_provides_main_loop(ui_plugin_t ui)
{
    /* we provide our own main loop - that of a gnome applet. */
    return 1;
}

/* execute the ui plugin's main loop - invoked only if */
/* 'ui_provides_main_loop' returned '1' previously.    */
void
ui_main_loop(ui_plugin_t ui)
{
    /* run a normal gnome applet main loop. */
    applet_widget_gtk_main();
}

/* register a callback with the ui plugin to be invoked */
/* every 'ms' milliseconds.                             */
/* invoked only if 'ui_provides_main_loop' returned '1' */
/* previously.                                          */
int
ui_register_timer_callback(ui_plugin_t ui,
			   int ms,
			   int (*cb)(void*),
			   void* data)
{
    ui->timer = gtk_timeout_add(ms, cb, data);

    return 1;
}

/* set the GUI's mode accordingh to ui->md and ui->bs. */
/* used when any of the states of r2l changes.         */
static
void
ui_refresh_r2l_mode(ui_plugin_t ui)
{
    if (ui->md == R2L_DISABLED) {
	if (ui->bs == BT_BASE_NEUTRAL) {
	    if (ui->thin_interface) {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_md_toggle)->child),
				  "0");
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_bs_toggle)->child),
				  "--");
	    }
	    else {
	        gtk_pixmap_set(ui->pixmap_state,
			       ui->gdk_pixmap_ln,
			       ui->gdk_mask_ln);
	    }
	}
	else if (ui->bs == BT_BASE_LTR) {
	    if (ui->thin_interface) {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_md_toggle)->child),
				  "0");
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_bs_toggle)->child),
				  "->");
	    }
	    else {
	        gtk_pixmap_set(ui->pixmap_state,
			       ui->gdk_pixmap_ll,
			       ui->gdk_mask_ll);
	    }
	}
	else if (ui->bs == BT_BASE_RTL) {
	    if (ui->thin_interface) {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_md_toggle)->child),
				  "0");
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_bs_toggle)->child),
				  "<-");
	    }
	    else {
	        gtk_pixmap_set(ui->pixmap_state,
			       ui->gdk_pixmap_lr,
			       ui->gdk_mask_lr);
	    }
	}
	gtk_widget_set_sensitive(ui->r2l_bs_toggle, FALSE);
    }
    else if (ui->md == R2L_ENABLED) {
	if (ui->bs == BT_BASE_NEUTRAL) {
	    if (ui->thin_interface) {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_md_toggle)->child),
				  "1");
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_bs_toggle)->child),
				  "--");
	    }
	    else {
	        gtk_pixmap_set(ui->pixmap_state,
			       ui->gdk_pixmap_rn,
			       ui->gdk_mask_rn);
	    }
	}
	else if (ui->bs == BT_BASE_LTR) {
	    if (ui->thin_interface) {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_md_toggle)->child),
				  "1");
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_bs_toggle)->child),
				  "->");
	    }
	    else {
	        gtk_pixmap_set(ui->pixmap_state,
			       ui->gdk_pixmap_rl,
			       ui->gdk_mask_rl);
	    }
	}
	else if (ui->bs == BT_BASE_RTL) {
	    if (ui->thin_interface) {
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_md_toggle)->child),
				  "1");
		gtk_label_set_text(GTK_LABEL(GTK_BIN(ui->r2l_bs_toggle)->child),
				  "<-");
	    }
	    else {
	        gtk_pixmap_set(ui->pixmap_state,
			       ui->gdk_pixmap_rr,
			       ui->gdk_mask_rr);
	    }
	}
	gtk_widget_set_sensitive(ui->r2l_bs_toggle, TRUE);
    }
}


/* notify the ui plugin about a mode change. */
void
ui_notify_of_r2l_mode(ui_plugin_t ui, R2L_STATE md, BIDITEXT_BASE_STATE bs)
{
    int should_refresh = 0;
    if (md != ui->md) {
	ui->md = md;
        should_refresh = 1;
    }
    if (bs != ui->bs) {
	ui->bs = bs;
	should_refresh = 1;
    }
    if (should_refresh) {
	ui_refresh_r2l_mode(ui);
    }
}

/* returns a file descriptor that the UI would like r2l to listen on. */
/* for the command-line plugin - this is stdin.			      */
int
ui_get_listening_fd(ui_plugin_t ui)
{
    return -1;
}

/* callback invoked when the user clicks the r2l_md_toggle button. */
void
r2l_md_toggle_cb(GtkToggleButton* r2l_toggle, void* data)
{
    ui_plugin_t ui = (ui_plugin_t)data;
    int rc;
    int should_refresh = 0;

    assert(ui);

    if (ui->md == R2L_ENABLED) {
	rc = r2l_disable(ui->r2l_ctrl);
	if (rc == 1) {
	    ui->md = R2L_DISABLED;
	    should_refresh = 1;
	}
    }
    else if (ui->md == R2L_DISABLED) {
	rc = r2l_enable(ui->r2l_ctrl);
	if (rc == 1) {
	    ui->md = R2L_ENABLED;
	    should_refresh = 1;
	}
    }
    if (should_refresh) {
	ui_refresh_r2l_mode(ui);
    }
}

/* callback invoked when the user clicks the r2l_bs_toggle button. */
void
r2l_bs_toggle_cb(GtkToggleButton* r2l_toggle, void* data)
{
    ui_plugin_t ui = (ui_plugin_t)data;
    int rc;
    int should_refresh = 1;

    assert(ui);

    if (ui->bs == BT_BASE_LTR) {
	rc = r2l_set_biditext_base_state_neutral(ui->r2l_ctrl);
	if (rc == 1) {
	    ui->bs = BT_BASE_NEUTRAL;
	    should_refresh = 1;
	}
    }
    else if (ui->bs == BT_BASE_NEUTRAL) {
	rc = r2l_set_biditext_base_state_rtl(ui->r2l_ctrl);
	if (rc == 1) {
	    ui->bs = BT_BASE_RTL;
	    should_refresh = 1;
	}
    }
    else if (ui->bs == BT_BASE_RTL) {
	rc = r2l_set_biditext_base_state_ltr(ui->r2l_ctrl);
	if (rc == 1) {
	    ui->bs = BT_BASE_LTR;
	    should_refresh = 1;
	}
    }
    if (should_refresh) {
	ui_refresh_r2l_mode(ui);
    }
}

/* callback invoked when r2l discovers the file descriptor of the UI */
/* is active. NOT USED in this plugin!                               */
int
ui_perform_callback(ui_plugin_t ui, r2llib_t r2l_ctrl)
{
    /* do nothing. */
    return 0;
}

/* cleanup the ui plugin. after this call, 'ui' may no longer be used. */
void
ui_cleanup(ui_plugin_t ui)
{
    fprintf(stdout, "%s: Gnome-Applet Plugin cleaning up\r\n", ui->progname);
    fflush(stdout);

    gtk_timeout_remove(ui->timer);

    free(ui->progname);
    free(ui);
}
