/*
 * $Id: access.c,v 1.1.1.1 2001/09/13 12:42:16 mulix Exp $
 */
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include "r2llib.h"
#include "access.h"

/** find the directory this file is in (we expect a fully qualified path name)
  * @return R2L_ERROR_TYPE (R2L_SUCCESS if ok, R2L_ERR_OOM if OOM and R2L_ERR_ACCESS
  * if we aren't allowed access 
  */
R2L_ERROR_TYPE
is_allowed_access(const char* fname)
{
     char* slash;
     char* dirname;
     int res;
     int is_dir; 
     struct stat buf;

     assert (fname);
     
     /* get a temporary buffer */
     dirname = strdup(fname);
     if (!dirname)
	  return R2L_ERR_OOM;

     /* get the directory name from the string */
     slash = rindex(dirname, '/');
     is_dir = !!slash; /* 1 if slash is non NULL, 0 is slash is NULL */

     /* get the directories component of the path we got */
     if (slash && (slash != dirname))
	  *slash = '\0';

     /* we now have a directory name, in the form '/path/to/dir', or 
	we have something of the form 'file_name' */
     if (is_dir){
	  res = stat(dirname, &buf);
	  if (res == -1)
	       goto no_access;

	  /* it exists, but is it a directory? */
	  if (!S_ISDIR(buf.st_mode))
	       goto no_access;
     }

     res = access(dirname, X_OK | W_OK | F_OK);
     if (res == -1){
	  goto no_access;
     }

     free(dirname);
     return R2L_SUCCESS;

 no_access:
     free(dirname);
     return R2L_ERR_ACCESS;
}
