/*
 * $Id: file_ops.c,v 1.1.1.1 2001/09/13 12:42:16 mulix Exp $
 */
/* for stat */
#include <sys/stat.h>
#include <unistd.h>

/* for strerror */
#include <string.h>

/* for errno */
#include <errno.h>

/* for remove */
#include <stdio.h>

#include <sys/types.h>
#include <fcntl.h>

#include "file_ops.h"

/* gets one char* parameter, the name of the file to check */
/* returns 1 if it exists, 0 if not, -1 on errors */
int
file_exists(const char* file_name)
{
     struct stat buf;
     int rc;
     
     /* check that the file name is valid */
     if (!file_name){
	  return -1;
     }
     
     rc = stat(file_name, &buf);
     if (rc != 0){
	  rc = 0;
     }else {
	  rc = 1;
     }
     
     return rc;
}

/* return 1 if the file was deleted, -1 on errors */
int 
delete_file(const char* file_name)
{
     int rc; 

     if (!file_name){
	  return -1;
     }

     rc = remove(file_name);
     if (rc != 0){
	  if (errno == ENOENT){
	       /* if the file doesnt exist, good for us */
	       return 1;
	  }
	  return -1;
     }

     return 1;
}

/* return the file descriptor ( value >= 0 ) on success and -1 on errors */
int
create_file(const char* file_name)
{
     /* anyone can read the file */
     mode_t create_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
     int rc;

     if (!file_name){
	  return -1;
     }

     rc = creat(file_name, create_mode);
     return rc;
}

/* return the file size, in bytes. -1 on error */
int 
file_size(const char* file_name)
{
     struct stat buf;
     int rc;
     
     /* check that the file name is valid */
     if (!file_name){
	  return -1;
     }
     
     rc = stat(file_name, &buf);
     if (rc == -1)
	  return -1;

     return buf.st_size;
}

int
create_file_with_size(const char* file_name, unsigned int size)
{
     int rc; 
     int fd;
     static char buf[16];

     /* sanitize size */
     size = ((size > 16) ? 16 : size);

     fd = create_file(file_name);

     if ((size == 0) || (fd == -1)){
	  return fd;
     }

     memset(buf, 'a', size);

     rc = write(fd, buf, size);
    
     if (rc < size){
	  close(fd);
	  delete_file(file_name);
	  return -1;
     }

     return fd;
}


#if 0 /* not used right now */
/* writes 'token' (a null terminated string) to the file 
   return 1 on succes, -1 on errors */
int 
write_file(const char* file_name, const char* token)
{
     int fd = -1;
     ssize_t rc;

     if ((!file_name)  || (!token))
	  return -1;

     if ((fd = create_file(file_name)) == -1)
	  return -1;

     /* we now have an fd, good, let's use it */
     rc = write(fd, token, strlen(token));
     
     /* close the file, we dont need this fd anymore */
     if (close(fd) == -1)
	  return -1;

     return (int)rc;
}

/* read at most 'bufsize' bytes into the buffer from the file 
   return the number of bytes read or -1 on failure
   NOTE: does NOT null terminate the buffer (unless the last byte read
   was null) */
int
read_file(const char* file_name, char* buffer, unsigned int bufsize)
{
     int fd;
     ssize_t rc;

     if ((!file_name) || (!buffer) || (!bufsize))
	  return -1;

     if ((fd = open(file_name, O_RDONLY)) == -1)
	  return -1;

     if ((rc = read(fd, (void*)buffer, bufsize)) == -1){
	  close(fd);
	  return -1;
     }

     return (int)rc;
}
#endif /* not used right now */
