/*
 * $Id: file_ops.h,v 1.1.1.1 2001/09/13 12:42:16 mulix Exp $
 */
#ifndef FILE_OPS_H_
#define FILE_OPS_H_

/** Internal utility file operations */

/**
 * Check if a file exists
 * @param file_name the name of the file to check
 * @return          1 if file_name exists, 0 if not, -1 on errors 
 */
int
file_exists(const char* file_name);

/**
 * Deletes a file
 * @param file_name the name of the file to delete
 * @return          1 if file_name was deleted, -1 on errors 
 */
int 
delete_file(const char* file_name);

/**
 * Creates a file
 * @param file_name the name of the file to create
 * @return the file descriptor ( value >= 0 ) on success and -1 on errors 
 */
int
create_file(const char* file_name);

/**
 * Get the size of a file
 * @param file_name the name of the file to check
 * @return the file size, in bytes. -1 on error 
 */
int 
file_size(const char* file_name);

/**
 * Create a file with size 'size'
 * @param file_name the name of the file to create
 * @return the file descriptor or -1 on error 
 */
int
create_file_with_size(const char* file_name, unsigned int size);


#if 0 /* not used right now */
/* writes 'token' (a null terminated string) to the file 
   return 1 on succes, -1 on errors */
int 
write_file(const char* file_name, const char* token); 

/* read at most 'bufsize' bytes into the buffer from the file 
   return the number of bytes read or -1 on failure
   NOTE: does NOT null terminate the buffer (unless the last byte read
   was null) */
int
read_file(const char* file_name, char* buffer, unsigned int bufsize);
#endif /* not used right now */


#endif /* FILE_OPS_H_ */
