/*
 * $Id: query_state.c,v 1.1.1.1 2001/09/13 12:42:16 mulix Exp $
 */
#include <assert.h>
#include <unistd.h>

#include "r2llib.h"
#include "query_state.h"
#include "file_ops.h"

void 
r2l_query_state_internal(const char* file_name, 
			 R2L_STATE* r2l_state, 
			 BIDITEXT_BASE_STATE* bt_state)
{
     R2L_STATE st  = R2L_ERROR; 
     BIDITEXT_BASE_STATE bt = BT_BASE_ERROR;

     /* from guy's email: 

	size:   r2l_mode      biditext base direction
	----------------------------------------------
	0       enabled       neutral
	1       disabled      neutral
	2       enabled       rtl
	3       disabled      rtl
	4       enabled       ltr
	5       disabled      ltr

     */

     int sz = file_size(file_name);

     /* we handle the 'file not found' case the same way as the size 1 case */
     if (sz == -1)
	  sz = 1;

     switch(sz){
     case(0):
	  st = R2L_ENABLED; bt = BT_BASE_NEUTRAL; break;
     case(1):
	  st = R2L_DISABLED;  bt = BT_BASE_NEUTRAL; break;
     case(2):
	  st = R2L_ENABLED; bt = BT_BASE_RTL; break;
     case(3):
	  st = R2L_DISABLED;  bt = BT_BASE_RTL; break;
     case(4):
	  st = R2L_ENABLED; bt = BT_BASE_LTR; break;
     case(5):
	  st = R2L_DISABLED;  bt = BT_BASE_LTR; break;
     default:
/*	  assert(sz * 0); */
	  st = R2L_ENABLED; bt = BT_BASE_NEUTRAL; break;
     }

     if (r2l_state)
	  *r2l_state = st;

     if (bt_state)
	  *bt_state = bt;
}


void r2l_set_state_internal(const char* file_name, 
			    R2L_STATE* st,
			    BIDITEXT_BASE_STATE* bt)
{
     unsigned int sz = 0;
     int fd; 

     assert(file_name && st && bt);

     if (st){
	  switch (*st){
	  case(R2L_ENABLED): /* do nothing */ 
	       break;
	  case(R2L_DISABLED): 
	       sz |= 0x0001; /* turn bit 1 on */
	       break;
	  case (R2L_ERROR): /* fallthrough */
	  default:
	       assert(*st * 0);
	  }
     }

     if (bt){
	  switch(*bt){
	  case(BT_BASE_NEUTRAL): /* do nothing */
	       break;
	  case (BT_BASE_RTL):
	       sz |= 0x0002; /* turn bit 2 on */
	       break;
	  case (BT_BASE_LTR):
	       sz |= 0x0004; /* turn bit 3 on */
	       break;
	  case (BT_BASE_ERROR): /* fallthrough */
	  default:
	       assert(*bt * 0);
	  }
     }
	  
     if ((fd = create_file_with_size(file_name, sz)) == -1){
	  goto error;
     }

     if (close(fd) == -1){
	  goto error;
     }

     return;     

 error:
     if (st)
	  *st = R2L_ERROR;

     if (bt)
	  *bt = BT_BASE_ERROR;
}
