#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <assert.h>

#include "r2l_state.h"
#include "file_ops.h"
#include "query_state.h"


R2L_STATE
set_r2l_state(const char* file_name, R2L_STATE requested_state)
{
     R2L_STATE state = R2L_ERROR;
     BIDITEXT_BASE_STATE bt = BT_BASE_NEUTRAL;

     if (!file_name || (requested_state == R2L_ERROR))
	  return R2L_ERROR;

     r2l_query_state_internal(file_name, &state, &bt);
     if (state == requested_state)
	  return state;

     /* we need to set the request r2l state, while maintaining the
	biditext base state we had */
     r2l_set_state_internal(file_name, &requested_state, &bt);
     return requested_state;
}

R2L_STATE
toggle_r2l_state(const char* file_name)
{
     R2L_STATE state = R2L_ERROR;

     if (!file_name)
	  return R2L_ERROR;

     state = get_r2l_state(file_name);
     
     switch (state){
     case (R2L_ENABLED):
	  state = set_r2l_state(file_name, R2L_DISABLED);
	  break;
     case (R2L_DISABLED):
	  state = set_r2l_state(file_name, R2L_ENABLED);
	  break;
     case (R2L_ERROR):
	  break;
     default:   
	  assert(0*state);
	  state = R2L_ERROR;
	  break;
     }
     
     return state;
}

R2L_STATE
get_r2l_state(const char* file_name)
{
     R2L_STATE st = R2L_ERROR;

     if (!file_name)
	  return R2L_ERROR;

     r2l_query_state_internal(file_name, &st, NULL);
     return st;
}
