/*
 * $Id: r2llib-test.c,v 1.1.1.1 2001/09/13 12:42:16 mulix Exp $
 */
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>

#include "r2llib.h"

const char* 
r2l_err_to_string(R2L_ERROR_TYPE err)
{
     switch(err){
     case(R2L_SUCCESS):           return "success";
     case(R2L_ERR_OOM):           return "out of memory";
     case(R2L_ERR_INVALID_REPR):  return "invalid representation";
     case(R2L_ERR_ACCESS):        return "no access";
     default:                     assert(0*err); return NULL;
     }
}

void 
test_access()
{
     r2llib_t rtl;
     R2L_ERROR_TYPE err;

     /* we should fail on this one */
     rtl = r2l_init_from_text_serialization("/no/such/dir/", &err);
     assert(!rtl && (err == R2L_ERR_ACCESS));

     /* we should fail on this one as well */
     rtl = r2l_init_from_text_serialization("./foo/bar/feez", &err);
     assert(!rtl && (err == R2L_ERR_ACCESS));

     /* and we should fell on this one as well! */
     rtl = r2l_init_from_text_serialization("./some_file_and_not_a_dir/.rev", &err);
     assert(!rtl && (err == R2L_ERR_ACCESS));

     /* we should succeed on this one */
     rtl = r2l_init_from_text_serialization("./.rev", &err);
     assert(rtl);
     r2l_destroy(rtl);
}

void
get_current_state(r2llib_t rtl)
{
     switch(r2l_get_current_r2l_state(rtl)){
     case(R2L_ENABLED):
	  fprintf(stdout, "r2l is hebrew\n"); break;
     case(R2L_DISABLED):
	  fprintf(stdout, "r2l is english\n"); break;
     default: 
	  fprintf(stdout, "r2l is unknown\n"); break;
     }

     switch(r2l_get_current_biditext_base_state(rtl)){
     case(BT_BASE_NEUTRAL):
	  fprintf(stdout, "biditext_base is neutral\n"); break;
     case(BT_BASE_RTL):
	  fprintf(stdout, "biditext_base is rtl\n"); break;
     case(BT_BASE_LTR):
	  fprintf(stdout, "biditext_base is ltr\n"); break;
     default:
	  fprintf(stdout, "biditext_base is unknown\n"); break;
     }
}

void
test_serialization(r2llib_t rtl)
{
     R2L_ERROR_TYPE err;
     const char* repr = r2l_get_text_serialization(rtl);
     r2llib_t soul = r2l_init_from_text_serialization(repr, &err);
     if (!soul){
	  fprintf(stderr, "r2l_init_from_text_repr failed, repr is '%s', err is '%s'\n",
		  repr, r2l_err_to_string(err));
	  return;
     }

     assert(r2l_get_current_r2l_state(rtl) == r2l_get_current_r2l_state(soul));

     r2l_destroy(soul);
}

int 
main(int argc, char* argv[])
{
     R2L_ERROR_TYPE err;
     r2llib_t rtl = r2l_init(&argc, argv, &err);

     if (!rtl){
	  fprintf(stderr, "r2l_init failed, err is '%s'\n", 
		  r2l_err_to_string(err));
	  exit(1);
     }

     test_serialization(rtl);

     test_access();

     fprintf(stdout, "--before starting\n");
     get_current_state(rtl);

     fprintf(stdout, "--enabling r2l\n");
     assert(r2l_enable(rtl) == 1);
     get_current_state(rtl);

     fprintf(stdout, "--disabling r2l\n");
     assert(r2l_disable(rtl) == 1);
     get_current_state(rtl);

     fprintf(stdout, "--toggling r2l\n");
     assert(r2l_toggle(rtl) == 1);
     get_current_state(rtl);

     fprintf(stdout, "--disabling r2l\n");
     assert(r2l_disable(rtl) == 1);
     get_current_state(rtl);

     fprintf(stdout, "--setting biditext base to ltr\n");
     assert(r2l_set_biditext_base_state_ltr(rtl) == 1);
     get_current_state(rtl);

     fprintf(stdout, "--setting biditext base to neutral\n");
     assert(r2l_set_biditext_base_state_neutral(rtl) == 1);
     get_current_state(rtl);

     fprintf(stdout, "--setting biditext base to rtl\n");
     assert(r2l_set_biditext_base_state_rtl(rtl) == 1);
     get_current_state(rtl);

     return 0;
}

