#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <unistd.h>

#include "refreshd_params.h"
#include "r2llib.h"


static void usage(int argc,char* argv[])
{
  fprintf(stderr,
	  "Usage: %s [--file-name <rev_file_name>] program [arguments]\n",
	  argv[0]);
}

int main(int argc,char* argv[])
{
  const char* bidiTextKey = NULL;
  r2llib_t r2llib = NULL;
  const char* errMsg = "No error";
  R2L_ERROR_TYPE err;
  char* oldPreload;
  char* preLoad = NULL;
  int preloadLength;

  if (argc < 2) {
    usage(argc,argv);
    exit(1);
  }

  r2llib = r2l_init(&argc,argv,&err);
  if (r2llib == NULL) {
    errMsg = "Cannot initialize r2l";
    goto err_r2llib_init;
  }

  bidiTextKey = r2l_get_text_serialization(r2llib);

  if (bidiTextKey == NULL) {
    errMsg = "Cannot get r2l token serialization";
    goto err_r2llib_get_serialization;
  }

  preloadLength = 0;
  oldPreload = getenv("LD_PRELOAD");
  
  if (oldPreload != NULL)
    preloadLength = strlen(oldPreload);

  preloadLength += sizeof(REFRESHD_HOOK_SO) + sizeof(BIDITEXT_SO) + 1;

  preLoad = malloc(preloadLength);

  if (preLoad == NULL) {
    errMsg = "Cannot allocate memory for LD_PRELOAD";
    goto err_malloc;
  }

  snprintf(preLoad,preloadLength,"%s %s",
	   REFRESHD_HOOK_SO,BIDITEXT_SO);

  if (oldPreload != NULL) {
    strcat(preLoad," ");
    strcat(preLoad,oldPreload);
  }

  setenv("LD_PRELOAD",preLoad,1);
  setenv(BIDITEXT_KEY_ENV_VAR,bidiTextKey,1);

  r2l_destroy(r2llib);
  free(preLoad);
  
  execvp(argv[1],&argv[1]);
  perror("execvp");
  
  exit(1);

 err_malloc:
 err_r2llib_get_serialization:
  r2l_destroy(r2llib);
 err_r2llib_init:
  fprintf(stderr,"Error: %s\n",errMsg);
  return 1;
}
