#ifndef DEBUG_H
#define DEBUG_H

#ifdef __cplusplus
extern "C" {
#endif

/**********************************************************************
 * This file contains the debugging features of refreshd. 
 * It's quite messy. Anyway, here is how to use it:
 * 1) If you want to get debug messages then you have to do the following
 *    a) Compile with -DNOISY
 *    b) #define the DEBUG_NOISE_LEVEL macro to one of the 
 *       NOISE_LEVEL_XXX values
 * 2) Debug messages are printed with the
 *       ERROR(), WARNING(), or MESSAGE() macros. They
 *       accept printf(3) like arguments, including syslog(3)'s %m 
 *       addition. man 3 syslog for more details
 * 3) All debug messages are at most MAX_LOG_LINE - 1 long including
 **********************************************************************/

#define MAX_LOG_LINE  160

#define NOISE_LEVEL_NONE    0
#define NOISE_LEVEL_ERROR   1
#define NOISE_LEVEL_WARNING 2
#define NOISE_LEVEL_MESSAGE 3

#define NOISE_LEVEL_MAX    32767 

  /* #define the macro below to one of the values above 
     The higher the value, the more noise you'll get  :-) */
#define DEBUG_NOISE_LEVEL  NOISE_LEVEL_MAX

#ifdef NOISY
#define NOISE_LEVEL DEBUG_NOISE_LEVEL
#else
#define NOISE_LEVEL NOISE_LEVEL_NONE
#endif

#if  NOISE_LEVEL == NOISE_LEVEL_MAX || NOISE_LEVEL >= NOISE_LEVEL_ERROR
# define ERROR log_error
#else
# define ERROR log_discard
#endif
 
#if NOISE_LEVEL == NOISE_LEVEL_MAX || NOISE_LEVEL >= NOISE_LEVEL_WARNING
# define WARNING log_warning
#else
# define WARNING log_discard
#endif
 
#if NOISE_LEVEL == NOISE_LEVEL_MAX || NOISE_LEVEL >= NOISE_LEVEL_MESSAGE
# define MESSAGE log_message
#else
# define MESSAGE log_discard
#endif


  void log_discard(const char* fmt, ...);
  void log_error(const char* fmt, ...);
  void log_warning(const char* fmt, ...);
  void log_message(const char* fmt, ...);

#ifdef __cplusplus
}
#endif

#endif
