#include <pthread.h>
#include <X11/Xlib.h>
#include <unistd.h>

#include "refreshd_params.h"
#include "traverse.h"
#include "debug.h"


pthread_cond_t refresh_cond = PTHREAD_COND_INITIALIZER;
pthread_mutex_t refresh_mutex = PTHREAD_MUTEX_INITIALIZER;


static WDB_IT_CALLBACK_RESULT 
refshBidiTextKeyItCallback(WindowDB::BidiTextKeyIterator bdtkit)
{
  r2lstate lastState = bdtkit.getLastState();
  r2lstate currentState = bdtkit.getCurrentState();

  //No change, go to the next key
  if (lastState == currentState)
    return WDB_IT_CONTINUE;

  bdtkit.setLastState(currentState);
  return WDB_IT_NOTHING;
}


static WDB_IT_CALLBACK_RESULT
refshWindowSetItCallback(WindowDB::BidiTextKeyIterator bdtkit,
			 WindowDB::DisplayMapIterator dpymit,
			 WindowDB::WindowSetIterator wsit)
{
  Display* dpy = (*dpymit).first->getDisplay();
  Window w = *wsit;

  MESSAGE("Refresh dpy: '%s' window id: 0x%lx\n",
	  DisplayString(dpy),w);

  XClearArea(dpy,w,0,0,0,0,True);
  
  return WDB_IT_NOTHING;
}



void* refresher(void* arg)
{
  for (;;) {
    pthread_mutex_lock(&refresh_mutex);
    pthread_cond_wait(&refresh_cond,&refresh_mutex);

    MESSAGE("Refreshing...\n");

    traverse(refshBidiTextKeyItCallback,
	     NULL,
	     refshWindowSetItCallback);

    pthread_mutex_unlock(&refresh_mutex);
  }
  /* not reached */
  return NULL;
}


void refreshNow(void)
{
  pthread_mutex_lock(&refresh_mutex);
  pthread_cond_signal(&refresh_cond);
  pthread_mutex_unlock(&refresh_mutex);
}


void* autoRefresher(void* arg)
{
  for(;;) {
    usleep(REFRESH_TIME);
    refreshNow();
  }
}
