#include <list>

#include "traverse.h"
#include "debug.h"

using namespace std;

extern WindowDB winDB;

static list<Window>  badList;

int refreshdXerrorHandler(Display* dpy,
			  XErrorEvent* ev)
{
  unsigned char errCode = ev->error_code;

  if (errCode == BadDrawable ||
      errCode == BadWindow) {
    Window badWindow = (Window)ev->resourceid;
    badList.push_back(badWindow);
    MESSAGE("%s:%d Info: Window 0x%lx moved on the bad list\n",
	    __FILE__, __LINE__, (unsigned long)badWindow);
  }
  else if (errCode == BadAtom) {
    Atom badAtom = (Atom)ev->resourceid;
    MESSAGE("%s:%d Info: Invalid atom: 0x%lx\n",
	    __FILE__,__LINE__,(unsigned long)badAtom);
  }
  else {
    WARNING("%s:%d Warning: unexpected error code (%u)\n",
	    __FILE__,__LINE__,(unsigned)errCode);
  }

  return 0;
}




void traverse(WdbBidiTextKeyItCallback bidiTextKeyItCallback,
	      WdbDisplayMapItCallback displayMapItCallback,
	      WdbWindowSetItCallback windowSetItCallback)
{
  WindowDB::BidiTextKeyIterator bdtkit, next_bdtkit;

  winDB.lock();
  
  for (bdtkit = winDB.bidiTextKeyBegin();
       bdtkit != winDB.bidiTextKeyEnd();
       bdtkit = next_bdtkit) {
    next_bdtkit = bdtkit;
    ++next_bdtkit;

    if (bidiTextKeyItCallback != NULL) {
      
      WDB_IT_CALLBACK_RESULT res = (*bidiTextKeyItCallback)(bdtkit);
      
      if (res == WDB_IT_CONTINUE)
	continue;
      else if (res == WDB_IT_BREAK)
	break;
      /*else 
	nothing;*/
    }

    WindowDB::DisplayMapIterator dpymit, next_dpymit;

    for (dpymit = winDB.dpyMapBegin(bdtkit);
	 dpymit != winDB.dpyMapEnd(bdtkit);
	 dpymit = next_dpymit) {
      next_dpymit = dpymit;
      ++next_dpymit;

      if (displayMapItCallback != NULL) {
      
	WDB_IT_CALLBACK_RESULT res = 
	  (*displayMapItCallback)(bdtkit,dpymit);
      
	if (res == WDB_IT_CONTINUE)
	  continue;
	else if (res == WDB_IT_BREAK)
	  break;
	/*else 
	  nothing;*/
      }   
       
      WindowDB::WindowSetIterator wsit,next_wsit;
      for ( wsit = winDB.wndsBegin(dpymit);
	    wsit != winDB.wndsEnd(dpymit);
	    wsit = next_wsit) {
	next_wsit = wsit;
	++next_wsit;

	if (windowSetItCallback != NULL) {
	  
	  WDB_IT_CALLBACK_RESULT res = 
	    (*windowSetItCallback)(bdtkit,dpymit,wsit);
      
	  if (res == WDB_IT_CONTINUE)
	    continue;
	  else if (res == WDB_IT_BREAK)
	    break;
	  /*else 
	    nothing;*/
	} 
      }
      XSync((*dpymit).first->getDisplay(),False);

      for (list<Window>::iterator liwi = badList.begin();
	   liwi != badList.end();) {
	winDB.remove(bdtkit,dpymit,*liwi);
	badList.erase(liwi++);
      }
    }
  }
  winDB.unlock();
}
      



