#ifndef TRAVERSE_H
#define TRAVERSE_H

#include <X11/Xlib.h>

#include "window_db.h"
#include "window_db_iterator.h"

/* error Handler */
int refreshdXerrorHandler(Display* dpy,
			  XErrorEvent* ev);

/* WindowDatabase traversal */

typedef enum {
  WDB_IT_NOTHING,   /* do nothing and continue with the computation */
  WDB_IT_CONTINUE,  /* issue a continue upon callback return */
  WDB_IT_BREAK      /* issue a break upon callback return */
} WDB_IT_CALLBACK_RESULT;

typedef WDB_IT_CALLBACK_RESULT 
(*WdbBidiTextKeyItCallback)(WindowDB::BidiTextKeyIterator bdtkit);

typedef WDB_IT_CALLBACK_RESULT
(*WdbDisplayMapItCallback)(WindowDB::BidiTextKeyIterator bdtkit,
			   WindowDB::DisplayMapIterator dpymit);

typedef WDB_IT_CALLBACK_RESULT
(*WdbWindowSetItCallback)(WindowDB::BidiTextKeyIterator bdtkit,
			  WindowDB::DisplayMapIterator dpymit,
			  WindowDB::WindowSetIterator wsit);


/* Iterate over the whole Window Database. 
   The callbacks are invoked for each level. 
   (bidiTextKeyItCallback is invoked in the outmost loop
   (biditext key level), displayMapItCallback() is invoked 
   in the intermediate (displayMap) level, while windowSetCallback
   is invoked at the innermost level.
   The iteration   continues according to the value returned by the 
   callback:
   1) If the callback returns WDB_IT_CONTINUE, the iteration
   skips to the next step on the same level
   2) If the callback returns WDB_IT_BREAK, the iteration breaks
   out of the current level. (not used right now)
   3) If the callback returns WDB_IT_NOTHING, the iteration proceeds
   as normal.
   Pasing a NULL for a callback means that no callback will be called
   (i.e. it is equivalent to a callback that doing nothing and
   returning WDB_IT_NOTHING) */

void traverse(WdbBidiTextKeyItCallback bidiTextKeyItCallback,
	      WdbDisplayMapItCallback displayMapItCallback,
	      WdbWindowSetItCallback windowSetItCallback);
	      


#endif  /* TRAVERSE_H */
