#ifndef WINDOW_DB_H
#define WINDOW_DB_H

#include <map>
#include <set>
#include <string>

#include <X11/Xlib.h>
#include <sys/types.h>
#include <pthread.h>

#include "r2llib.h"

#include "auto_display.h"
#include "r2lstate.h"


class WindowDBBidiTextKeyIterator;

class WindowDB {
public:
  typedef std::set<Window> WindowSet;
  typedef std::map<AutoDisplay*,WindowSet,AutoDisplayCompare> DisplayMap;


  typedef WindowDBBidiTextKeyIterator BidiTextKeyIterator;
  typedef DisplayMap::iterator DisplayMapIterator;
  typedef WindowSet::iterator WindowSetIterator;

  enum Result {
    SUCCESS,
    FAILURE
  };
  
  WindowDB(void);
  ~WindowDB(void);

  Result add(const std::string& dispName,
	     Window w,
	     const std::string& bidiTextKey);


  Result remove(BidiTextKeyIterator& bidiTextKeyIt,
		DisplayMapIterator& dpyIt,
		Window w);
  

  void lock(void);
  void unlock(void);

  BidiTextKeyIterator bidiTextKeyBegin();
  BidiTextKeyIterator bidiTextKeyEnd();
   
  DisplayMapIterator dpyMapBegin(BidiTextKeyIterator& current);
  DisplayMapIterator dpyMapEnd(BidiTextKeyIterator& current);

  WindowSetIterator wndsBegin(DisplayMapIterator& current);
  WindowSetIterator wndsEnd(DisplayMapIterator& current);
  

private:
  struct PerBidiTextKeyData {
    DisplayMap dpyMap;
    r2llib_t r2llib;
    r2lstate lastState;
  public:
    PerBidiTextKeyData() 
      : r2llib(NULL) {}
  };

  typedef std::map<std::string,PerBidiTextKeyData> BidiTextKeyData;

  BidiTextKeyData m_bidiTextKeys;
  pthread_mutex_t m_dbLock;
  bool m_isLocked;

  friend class WindowDBBidiTextKeyIterator;
private:
  //Disallowed
  WindowDB(const WindowDB &);
  WindowDB & operator = (const WindowDB &);
};

#endif /* WINDOW_DB_H */
