#ifndef MESSAGES_H
#define MESSAGES_H

#include <X11/Xlib.h>
#include <sys/types.h>
#include "refreshd_params.h"



/*Local addresses have the following format:
  /tmp/refreshd_PPPPP_SSSSS_UUUUU_RRRRR

  where   PPPPP = pid
          SSSSS = timestamp (seconds)
          UUUUU = timestamp (microseconds)
          RRRRR = random number

The hope is to reduce to minimum the collisions.

*/

#define LOCAL_ADDR_PREFIX   "/tmp/refreshd"


/* Space to store a textual representation of an integer type */

#define TEXTSIZEOF(i)   (4*sizeof(i) + 1)

#define MAX_ADDRESS_SIZE (sizeof(LOCAL_ADDR_PREFIX) + \
                          TEXTSIZEOF(int) + \
                          1 + \
                          TEXTSIZEOF(long) + \
                          1 + \
                          TEXTSIZEOF(long) + \
			  1 + \
			  TEXTSIZEOF(int)) + \
                          1

/* 
  "SUSv2 guarantees that `Host names are limited to 255 bytes'"
  from gethostname(2) man page
*/
#define MAX_HOST_NAME_LENGTH  256

/* At most a display name should consist of
   hostname::disp.screen
  (double colon is used for DECNet only. Hmm what about
  hostnames on DECNet??
*/

#define MAX_DISP_NAME   MAX_HOST_NAME_LENGTH + \
                        2 + \
			TEXTSIZEOF(unsigned short) + \
			1 + \
			TEXTSIZEOF(unsigned short) + \
			1



typedef enum {
  REFRESHD_PING,
  REFRESHD_REPLY_PING, 
  
  REFRESHD_ADD_WINDOW,
  REFRESHD_REPLY_ADD_WINDOW,
  
  /*Server does not reply for this message*/
  REFRESHD_REFRESH_NOW

} REFRESHD_MESSAGE_TYPE;



typedef struct {
  pid_t pid;
  Window w;
  Atom bidiTextKeyAtom;
  char dispName[MAX_DISP_NAME];
} AddWindowMessageData;

typedef struct {
  pid_t pid;
} ReplyAddWindowMessageData;


typedef struct {
  pid_t pid;
} PingMessageData;


typedef struct {
  pid_t pid;
} ReplyPingMessageData;


typedef struct {
  REFRESHD_MESSAGE_TYPE type;
  union {
    AddWindowMessageData addWindowMsg_;
    ReplyAddWindowMessageData replyAddWindowMsg_;
    PingMessageData pingMsg_;
    ReplyPingMessageData replyPingMsg_;
  }u_;
#define addWindowMsg u_.addWindowMsg_
#define replyAddWindowMsg u_.replyAddWindowMsg_
#define pingMsg u_.pingMsg_
#define replyPingMsg u_.replyPingMsg_

} Message;


#endif  /*MESSAGES_H*/
