#ifndef REFRESHD_CNT_H
#define REFRESHD_CNT_H

#include <sys/types.h>
#include <X11/Xlib.h> /* For Window */

/* This library is intended to be used for communicating 
   with the refresh daemon */


#ifdef __cplusplus
extern "C" {
#endif

  typedef enum {
    RD_CNT_SUCCESS,
    RD_CNT_UNINITIALIZED_ERROR,
    RD_CNT_NO_DAEMON_ERROR,
    RD_CNT_INVALID_BIDITEXT_KEY_ERROR,
    RD_CNT_INVALID_DISPLAY_ERROR,
    RD_CNT_SOCKET_ERROR,
    RD_CNT_SEND_ERROR,
    RD_CNT_RECV_ERROR,
    RD_CNT_ADDRESS_TOO_LONG_ERROR,
    RD_CNT_PROTOCOL_ERROR,
    RD_CNT_UNKNOWN_ERROR
  } RD_CNT_STATUS;
  
  /* Must be called before everything else */
  RD_CNT_STATUS rdCntInitialize(void);

  /* Must be called after everything else */
  void rdCntFinalize(void);

  /* 
     returns an error message corresponding to status.
     The returned value points to a statically allocated
     area, so it should not be freed.
   */
  const char* rdCntGetErrorMessage(RD_CNT_STATUS status);

  /* check if the daemon still responds */
  RD_CNT_STATUS rdCntPingDaemon(void);

  /* 
     cause the daemon to refresh all the windows whose
     biditext key value has changed (i.e. windows that
     have to be refreshed. To be used by GUIs if the refresh
     daemon has been started with --no_auto
  */
  RD_CNT_STATUS rdCntRefreshNow(void);


  /* 
     adds a window to the refresh daemon database. Intended to
     be used by refreshd_hook only.
   */
  RD_CNT_STATUS rdCntAddWindow(Display* dpy,
			       Window w,
			       const char* bidiTextKey);

  
#ifdef __cplusplus
}
#endif

#endif  /* REFRESHD_CNT_H */
