#include <stdlib.h>
#include <stdio.h>

#include <sys/types.h>
#include <unistd.h>

#include <dlfcn.h>
#include <X11/Xlib.h>
#include "refreshd_cnt.h"
#include "refreshd_params.h"


typedef Window (*XCreateWindowPtrType)(Display*		/* display */,
				       Window		/* parent */,
				       int			/* x */,
				       int			/* y */,
				       unsigned int	/* width */,
				       unsigned int	/* height */,
				       unsigned int	/* border_width */,
				       int			/* depth */,
				       unsigned int	/* class */,
				       Visual*		/* visual */,
				       unsigned long	/* valuemask */,
				       XSetWindowAttributes* /* attributes */
				       ); 

static XCreateWindowPtrType XCreateWindowPtr;

typedef Window (*XCreateSimpleWindowPtrType)(Display*	/* display */,
					     Window	/* parent */,
					     int	/* x */,
					     int	/* y */,
					     unsigned int  /* width */,
					     unsigned int  /* height */,
					     unsigned int  /* border_width */,
					     unsigned long /* border */,
					     unsigned long /* background */
					     );
static XCreateSimpleWindowPtrType XCreateSimpleWindowPtr;




static int libraryInitialized = 0;

static const char* bidiTextKey;

static int initLibrary(void)
{
  void* xLibHandle;
  const char* errMsg = "No Error";
  RD_CNT_STATUS res;

  res = rdCntInitialize();
  if (res != RD_CNT_SUCCESS) {
    errMsg = rdCntGetErrorMessage(res);
    goto err_rd_cnt_init;
  }

  xLibHandle = dlopen("libX11.so",RTLD_NOW);
  if (xLibHandle == NULL) {
    errMsg = dlerror();
    goto err_dlopen_xlib;
  }

  XCreateWindowPtr = (XCreateWindowPtrType)dlsym(xLibHandle,"XCreateWindow");
  if (XCreateWindowPtr == NULL) {
    errMsg = dlerror();
    goto err_dlsym;
  } 

  XCreateSimpleWindowPtr = 
    (XCreateSimpleWindowPtrType)dlsym(xLibHandle,
				      "XCreateSimpleWindow");
  if (XCreateSimpleWindowPtr == NULL) {
    errMsg = dlerror();
    goto err_dlsym;
  } 


  if ((bidiTextKey = getenv(BIDITEXT_KEY_ENV_VAR)) == NULL) {
    errMsg = "Environment variable " BIDITEXT_KEY_ENV_VAR  
      " undefined.\n refreshd cannot run\n";
    goto err_env_var;
  }

  return 1;

 err_env_var:
 err_dlsym:
  dlclose(xLibHandle);
 err_dlopen_xlib:
  rdCntFinalize();
 err_rd_cnt_init:
  fprintf(stderr,"Error: %s\n",errMsg);
  return 0;
}


static void checkInit(void)
{

  if (!libraryInitialized) {
    fprintf(stderr,"Error: refreshd_hook initialization failed\n");
    exit(1);
  }
}




void _init(void)
{
  if (initLibrary())
    libraryInitialized = 1;
}

void _fini(void)
{
  rdCntFinalize();
}



Window XCreateWindow(Display* display,
		     Window parent,
		     int x,
		     int y,
		     unsigned int width,
		     unsigned int height,
		     unsigned int border_width,
		     int depth,
		     unsigned int class__,
		     Visual* visual,
		     unsigned long valuemask,
		     XSetWindowAttributes* attributes)
{
  Window wnd;

  checkInit();

  wnd = (*XCreateWindowPtr)(display,
			    parent,
			    x,y,
			    width,height,
			    border_width,
			    depth,
			    class__,
			    visual,
			    valuemask,
			    attributes);


  rdCntAddWindow(display,
		 wnd,
		 bidiTextKey);

  return wnd; 
}



Window XCreateSimpleWindow(Display* display ,
			   Window parent,
			   int x ,
			   int y ,
			   unsigned int width,
			   unsigned int	height,
			   unsigned int border_width,
			   unsigned long border,
			   unsigned long background)
{
  Window wnd;

  checkInit();

  wnd = (*XCreateSimpleWindowPtr)(display ,
				  parent ,
				  x,y,
				  width,height ,
				  border_width ,
				  border ,
				  background);

  rdCntAddWindow(display,
		 wnd,
		 bidiTextKey);
  
  
  return wnd;
}

