#include <iostream>
#include <string>

#include <syslog.h>

#include "window_db.h"
#include "window_db_iterator.h"

using namespace std;

static void dump(WindowDB* wdb)
{
  for (WindowDB::BidiTextKeyIterator bdtki = wdb->bidiTextKeyBegin();
       bdtki != wdb->bidiTextKeyEnd();
       ++bdtki) {
    cout << "BidiTextKey: " << bdtki.getBidiTextKeyName() << endl;

    for (WindowDB::DisplayMapIterator dpyit = wdb->dpyMapBegin(bdtki);
	 dpyit != wdb->dpyMapEnd(bdtki);
	 ++dpyit) {
      cout << "\tDisplayName: " 
	   << DisplayString((*dpyit).first->getDisplay()) << endl;

      for (WindowDB::WindowSetIterator wndsit = wdb->wndsBegin(dpyit);
	   wndsit != wdb->wndsEnd(dpyit);
	   ++wndsit) {
	cout << "\t\tWindow:" << *wndsit << endl;
      }
    }
  }
}


static void remove(WindowDB* wdb,
		   const string& bidiTextKey,
		   const string& dpyName,
		   Window wnd)
{
  WindowDB::BidiTextKeyIterator bdtki,next_bdtki;
  WindowDB::DisplayMapIterator dpyit,next_dpyit;
  WindowDB::WindowSetIterator wndsit,next_wndsit;
  
  for ( bdtki = wdb->bidiTextKeyBegin();
	bdtki != wdb->bidiTextKeyEnd();
	bdtki = next_bdtki) {

    //bdtki might become invalid after deletion
    next_bdtki = bdtki;
    ++next_bdtki;


    if (bdtki.getBidiTextKeyName() != bidiTextKey)
       continue;

    for (dpyit = wdb->dpyMapBegin(bdtki);
	 dpyit != wdb->dpyMapEnd(bdtki);
	 dpyit = next_dpyit) {
      next_dpyit = dpyit;
      ++next_dpyit;

      if (DisplayString((*dpyit).first->getDisplay()) != dpyName)
	continue;

      for (wndsit = wdb->wndsBegin(dpyit);
	   wndsit != wdb->wndsEnd(dpyit);
	   wndsit = next_wndsit) {
	next_wndsit = wndsit;
	++next_wndsit;

	if (*wndsit != wnd)
	  continue;
	
	WindowDB::Result res = wdb->remove(bdtki,dpyit,wnd);

	if (res == WindowDB::SUCCESS) {
	  cout << "Removed ok" << endl;
	}
	else {
	  cout << "Remove failed" << endl;
	}
      }
    }
  } 
}


static void help(void)
{
  cout << "a[dd]  <dpyName> <windowId> <bidiTextKeyName>\n";
  cout << "p[rint]\n";
  cout << "r[emove] <dpyName> <windowId> <bidiTextKeyName>\n";
  cout << "[q]uit\n";
  cout << "[h]elp\n";
  cout << flush;
}

int main()
{
  WindowDB* wdb = new WindowDB;
  const string prompt = "testDB>";
  char cmd;


  openlog("Test Window DB",LOG_PERROR,LOG_DAEMON);

  help();

  while (cin) {
    cout << prompt <<flush;
    cin >> cmd;

    switch(cmd) {
    case 'a':
    case 'A':
      {
	string dpyName;
	Window w;
	string bidiTextKeyName;

	cin >> dpyName >> w >>  bidiTextKeyName;
	
	wdb->lock();
	wdb->add(dpyName,w,bidiTextKeyName);
	wdb->unlock();
      }
      break;
      
    case 'p':
    case 'P':
      dump(wdb);
      break;

    case 'h':
    case 'H':
      help();
      break;

    case 'r':
    case 'R':
      {
	string bidiTextKeyName;
	Window wnd;
	string dpyName;

	cin >> dpyName >> wnd >> bidiTextKeyName; 
	wdb->lock();
	remove(wdb,bidiTextKeyName,dpyName,wnd);
	wdb->unlock();
      }
      break;

    case 'q':
    case 'Q':
      goto out;
      break;

    default:
      if (isspace(cmd))
	continue;

      cerr << "Illegal character " << "'" << cmd << "'" <<endl;
      break;
    }
  }

 out:
  closelog();
  delete wdb;
  return 0;
}
